<?php

declare( strict_types=1 );

namespace RankBear\Admin;

use RankBear\Common\Assets as Common_Assets;
use RankBear\Common\Common as Common;
use RankBear\Common\RestApi;
use RankBear\Common\Settings\Choices;
use RankBear\Common\Settings\Main as Common_Settings;
use RankBear\Common\Utilities\Http;
use RankBear\PluginData as PluginData;
use RankBear\RbApi;
use RankBear\RbFreemius;
use WP_Post;

// Abort if this file is called directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( Assets::class ) ) {
	/**
	 * Enqueues the global admin assets.
	 *
	 * Settings Page adds additional.
	 */
	class Assets {

		/**
		 * @var Common_Assets
		 */
		var $common_assets;

		/**
		 * The Common instance.
		 *
		 * @var Common
		 */
		public $common;

		/**
		 * Get the Settings instance from Common.
		 *
		 * @var Common_Settings
		 */
		private $settings;

		public function __construct() {
			$this->common_assets = new Common_Assets();
			$this->common   = new Common();
			$this->settings = new Common_Settings();
		}

		/**
		 * Get this plugin's Admin images base path, including trailing slash.
		 *
		 * Example: /app/public/wp-content/plugins/rankbear/src/Admin/images/
		 *
		 * @return string
		 */
		public static function admin_images_dir_path(): string {
			return PluginData::plugin_dir_path()
				. 'src'
				. DIRECTORY_SEPARATOR
				. 'Admin'
				. DIRECTORY_SEPARATOR
				. 'images'
				. DIRECTORY_SEPARATOR;
		}

		/**
		 * Get this plugin's Admin images base URL, including trailing slash.
		 *
		 * Example: https://example.com/wp-content/plugins/rankbear/src/Admin/images/
		 *
		 * @return string
		 */
		public static function admin_images_dir_url(): string {
			return PluginData::plugin_dir_url() . 'src/Admin/images/';
		}

		/**
		 * Returns the source code of an SVG file, optionally base64 encoded.
		 *
		 * @param string $file_name
		 * @param bool   $base64    Whether to return base64 encoded output.
		 *
		 * @return string
		 */
		public function get_svg_file_source(
			string $file_name = 'rankbear',
			bool $base64 = false
		) {
			$svg = file_get_contents( self::admin_images_dir_path() . $file_name . '.svg' );

			if ( false === $svg ) {
				return '';
			}

			if ( $base64 ) {
				$svg = 'data:image/svg+xml;base64,' . base64_encode( $svg );
			}

			return $svg;
		}

		/**
		 * Add the Settings page to the wp-admin menu.
		 */
		public function add_plugin_admin_menu(): void {
			if ( ! current_user_can( $this->common->required_capability() ) ) {
				return;
			}

			$hook_suffix = add_menu_page(
				PluginData::get_plugin_display_name(),
				PluginData::get_plugin_display_name(),
				$this->common->required_capability(),
				$this->settings->get_settings_page_slug(),
				[ new Main(), 'settings_page' ],
				$this->get_svg_file_source( 'up-arrow-circled', true ),
				110
			);

			// Empty if insufficient permissions. We don't want our data put to page source in that case (but the action would not fire successfully anyway).
			if ( ! empty( $hook_suffix ) ) {
				add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_settings_page_assets' ] );
				add_action( "admin_print_scripts-{$hook_suffix}", [ $this, 'localize_scripts' ] );
			}
		}

		/**
		 * Register all the Admin stylesheets.
		 */
		public function register_styles(): void {
			$shared_deps = [
				'wp-components',
			];

			$this->common_assets->register_style(
				'admin-settings',
				'',
				$shared_deps
			);

			$this->common_assets->register_style(
				'admin-meta-box',
				'',
				$shared_deps
			);
		}

		/**
		 * Register all the Admin scripts.
		 *
		 * @see wp_default_packages_vendor()
		 */
		public function register_scripts(): void {
			$shared_deps = [
				'moment',
				'wp-api',
				'wp-api-fetch',
				'wp-components',
				'wp-data',
				'wp-element',
				'wp-i18n',
				'wp-notices',
				'wp-url',
			];

			$this->common_assets->register_script(
				'admin-settings',
				'',
				$shared_deps
			);

			$this->common_assets->register_script(
				'admin-meta-box',
				'',
				$shared_deps + [ 'wp-editor' ]
			);
		}

		/**
		 * Enqueue the Settings Page assets if we should.
		 */
		public function enqueue_settings_page_assets(): void {
			if (
				current_user_can( $this->common->required_capability() )
				&& ( new Main() )->is_our_settings_page()
			) {
				$this->common_assets->enqueue_style( 'admin-settings' );
				$this->common_assets->enqueue_script( 'admin-settings' );
			}
		}

		/**
		 * Enqueue the Meta Box assets if we should.
		 */
		public function enqueue_meta_box_assets(): void {
			if ( ( new Main() )->is_ready_for_full_meta_box_content() ) {
				$this->common_assets->enqueue_style( 'admin-meta-box' );
				$this->common_assets->enqueue_script( 'admin-meta-box' );
				add_action( 'admin_print_scripts-post.php', [ $this, 'localize_scripts' ] );
			}
		}

		/**
		 * Get the current post's REST API response data.
		 *
		 * @return array|null
		 */
		private function get_current_post_rest_api_response_data(): ?array {
			$current_post = get_post();

			if ( ! $current_post instanceof WP_Post ) {
				return null;
			}

			return ( new RestApi() )->get_single_post_response_data( $current_post->ID );
		}

		/**
		 * Localize the Admin's JavaScripts.
		 */
		public function localize_scripts() {
			$api = new RbApi();

			$site_id = $api->get_site_id();

			if( empty( $site_id ) ) {
				return;
			}

			$rb_freemius = new RbFreemius();

			$current_post_data = $this->get_current_post_rest_api_response_data();

			$choices = new Choices();

			$data = [
				// The CSS ID into which our React app inserts its content.
				'entryId'       => PluginData::js_entry_id(),
				'currentPost'   => [
					// Can't rely on `wp.data.select("core/editor").getCurrentPostId()` because that doesn't work in Classic Editor and could be non-public/non-published
					'id' => ! empty( $current_post_data['id'] ) ? $current_post_data['id'] : null,
				],
				'domainSlug'    => ( new Http() )->get_sluggified_host_domain(),
				'pluginInfo'    => [
					'name'          => PluginData::get_plugin_display_name(),
					'version'       => PluginData::plugin_version(),
					'settingsTitle' => $this->settings->get_settings_title(),
				],
				// The root location where we store images specific to the Admin area.
				'imagesBaseUrl' => self::admin_images_dir_url(),
				'api'           => [
					'base'   => $api::get_base_url(),
					'siteId' => $site_id,
				],
				'plan'          => [
					'is_paying_or_trial' => $rb_freemius->freemius_global()->is_paying_or_trial(),
					'upgrade_url' => $rb_freemius->freemius_global()->_get_admin_page_url( 'pricing' ),
				],
				'optionsInfo'   => [
					/**
					 * The option prefix, in case we want to do any filtering for just our stuff.
					 * Access it in JS via `rankBearScriptData.optionsInfo.prefix`.
					 *
					 * @see \RankBear\Common\Settings\Main::get_option_prefix()
					 */
					'prefix' => $this->settings->get_option_prefix(),
				],
				'initialValues' => [
					/**
					 * React-Select doesn't re-render from useState() so we need to set it as useState()'s initial value.
					 * Access it in JS via `rankBearScriptData.initialValues.se_country_initial`.
					 *
					 * @see \RankBear\Common\Settings\Main::get_option_prefix()
					 */
					'se_country_initial' => $this->settings->get_option( 'se_country' ),
				],
				'choicesFor'    => [
					'se_country' => $choices->get_search_engine_countries( true ),
				],
			];

			wp_localize_script(
				PluginData::get_asset_handle( 'admin-settings' ),
				'rankBearScriptData',
				$data
			);

			wp_localize_script(
				PluginData::get_asset_handle( 'admin-meta-box' ),
				'rankBearScriptData',
				$data
			);
		}
	}
}