<?php

declare( strict_types=1 );

namespace RankBear\Admin;

use Freemius;
use RankBear\Common\Common as Common;
use RankBear\Common\Settings\Main as Common_Settings;
use RankBear\Common\Utilities\Http;
use RankBear\Common\Utilities\Posts;
use RankBear\Common\Utilities\Strings;
use RankBear\PluginData as PluginData;
use RankBear\RbFreemius;
use WP_Screen;

// Abort if this file is called directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( Main::class ) ) {
	/**
	 * The admin-specific settings.
	 */
	class Main {

		/**
		 * The Common instance.
		 *
		 * @var Common
		 */
		public $common;

		/**
		 * Get the Settings instance from Common.
		 *
		 * @var Common_Settings
		 */
		private $settings;

		/**
		 * Initialize the class and set its properties.
		 */
		public function __construct() {
			$this->common   = new Common();
			$this->settings = new Common_Settings();
		}

		/**
		 * Add Settings link within Plugins List page.
		 *
		 * @param array $links
		 *
		 * @return array
		 */
		public function customize_action_links( array $links ): array {
			$link_to_settings_page = sprintf(
				'<a href="%s">%s</a>',
				esc_url( $this->settings->get_main_settings_page_url() ),
				$this->settings->get_settings_word()
			);

			$custom_action_links = [
				$link_to_settings_page,
			];

			return array_merge( $custom_action_links, $links );
		}

		/**
		 * Get the settings page ID, which is added as a body.class and is the $hook_suffix passed to 'admin_enqueue_scripts'.
		 *
		 * If you add your page as a submenu to anything other than "Settings", such as to be a top-level menu or
		 * submenu of a Custom Post Type, you'll need to edit the hard-coded part of this function.
		 *
		 * @see get_plugin_page_hookname()
		 *
		 * @return string
		 */
		public function get_settings_page_id(): string {
			return 'toplevel_page_' . $this->settings->get_settings_page_slug();
		}

		/**
		 * Detect if we are on our Settings Page.
		 *
		 * @return bool
		 */
		public function is_our_settings_page(): bool {
			$current_screen = get_current_screen();

			$rb_freemius = ( new RbFreemius() )->freemius_global();

			/**
			 * We don't want to load on Freemius-only pages, since none of our Dashboard is displayed and don't want
			 * styling or console error from our JS var not existing. Contact and Pricing pages don't load our stuff
			 * because they're wrappers for content loaded from Freemius.
			 */
			$current_page = ( new Http() )->get_request_param( 'page' );
			$base_page = $this->settings->get_settings_page_slug();

			if (
				$current_screen instanceof WP_Screen
				&& $rb_freemius instanceof Freemius
				&& $rb_freemius->is_product_settings_page()
				&& ( new Strings() )->stringy( $current_page )->isEqualsCaseInsensitive( $base_page )
			) {
				return true;
			}

			return false;
		}

		/**
		 * Change the <head> title (browser tab hover) to our settings page title.
		 *
		 * @param string $admin_title The page title, with extra context added.
		 *
		 * @return string
		 */
		public function override_settings_page_title( string $admin_title): string {
			if ( $this->is_our_settings_page() ) {
				return $this->settings->get_settings_title();
			}

			return $admin_title;
		}

		/**
		 * If we're currently on a single post that is in a public (publish) status (or 'future'), since we may want to
		 * setup tracking now for a post to be published in 3 days.
		 *
		 * @return bool
		 */
		public function is_ready_for_full_meta_box_content(): bool {
			$current_screen = get_current_screen();

			if (
				! $current_screen instanceof WP_Screen
				|| 'post' !== $current_screen->base
				|| ! ( new Posts() )->is_post_in_public_status( null, true )
			) {
				return false;
			}

			return true;
		}

		/**
		 * Output the JavaScript entry point or <noscript>.
		 */
		public function print_js_entry_with_noscript(): void {
			$noscript_message = sprintf(
				esc_html_x(
					'Please %1$senable JavaScript%2$s to make sure the %3$s plugin works correctly.',
					'noscript message'
				),
				'<a href="https://www.enable-javascript.com/" target="_blank" rel="noopener noreferrer">',
				'</a>',
				PluginData::get_plugin_display_name()
			);

			printf(
				'<div class="wrap">
					<div class="hide-if-no-js" id="%s"></div>
					<div class="hide-if-js notice notice-error" style="margin-left: 2px;">
						<noscript><p>%s</p></noscript>
					</div>
				</div>',
				PluginData::js_entry_id(),
				$noscript_message
			);
		}

		/**
		 * Outputs HTML for the plugin's Settings page.
		 */
		public function settings_page(): void {
			if ( ! current_user_can( $this->common->required_capability() ) ) {
				wp_die( esc_html__( 'You do not have sufficient permissions to access this page.', 'rankbear' ) );
			}

			$this->print_js_entry_with_noscript();
		}

		/**
		 * Get the meta box priority to determine which meta boxes it loads above/below.
		 */
		private function get_meta_box_priority(): string {
			$priority = 'default';

			/**
			 * The meta box priority affects which meta boxes it loads above/below.
			 *
			 * @since 1.0.0
			 *
			 * @param string $priority 'default', 'high', or 'low'.
			 *
			 * @return string
			 */
			apply_filters( PluginData::plugin_text_domain_underscores() . '_meta_box_priority', $priority );

			return $priority;
		}

		/**
		 * Add the post editor meta box.
		 *
		 * If Freemius isn't setup/connected, won't render anything.
		 */
		public function add_meta_box(): void {
			if (
				! current_user_can( $this->common->required_capability() )
				|| ! ( new RbFreemius() )->site_is_connected()
			) {
				// @todo Could link back to RankBear settings screen to encourage Freemius setup to be able to use the plugin.
				return;
			}

			add_meta_box(
				PluginData::plugin_text_domain() . '-meta-box',
				sprintf(
					esc_html_x( '%s Keyword Tracking', 'meta box title' ),
					PluginData::get_plugin_display_name()
				),
				[ $this, 'do_meta_box' ],
				array_keys(
					( new Posts() )->get_public_post_types()
				),
				'advanced',
				$this->get_meta_box_priority(),
				[ '__block_editor_compatible_meta_box' => true ]
			);
		}

		/**
		 * Output the meta box content.
		 *
		 * Freemius setup check already passed before we got here.
		 * This method's output could be a notice that the post isn't published/scheduled yet (so we can't submit to
		 * tracking) or the JS entry point.
		 *
		 * @todo respond to post status changes (only possible within block editor?) without requiring manual page reload.
		 */
		public function do_meta_box() {
			if ( ! $this->is_ready_for_full_meta_box_content() ) {

				// Emulate `.notice.notice-info` but can't add those classes within block editor.
				printf(
					'<div style="
background: #fff;
border: 1px solid #ccd0d4;
border-left-width: 4px;
box-shadow: 0 1px 1px rgba(0,0,0,.04);
margin: 15px;
margin-bottom: 2px;
padding: 1px 12px;
border-left-color: #00a0d2;"><p>%s</p></div>',
					esc_html_x(
						"Because we need a post's ID and finalized Permalink, we only support displaying and adding keywords on posts in a Public or Future status. Publish (or schedule) this post, refresh the editor, and then you will be able to view and edit its keyword tracking.",
						'meta box on Add New Post'
					)
				);

				return;
			}

			$this->print_js_entry_with_noscript();
		}

	}
}
