<?php

namespace RankBear;

// Abort if this file is called directly.
use RankBear\Core as Core;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( Bootstrap::class ) ) {
	/**
	 * The file that gets things going.
	 */
	class Bootstrap {
		/**
		 * Begins execution of the plugin.
		 *
		 * Since everything within the plugin is registered via hooks, then kicking off the plugin from this point in the file
		 * does not affect the page life cycle.
		 *
		 * Also returns copy of the app object so 3rd party developers can interact with the plugin's hooks contained within.
		 *
		 * @return Bootstrap|null
		 */
		public function init(): ?self {
			$plugin = new self();

			if ( $plugin->is_ready() ) {
				$core = new Core\Init();
				$core->run();

				return $plugin;
			} else {
				return null;
			}
		}

		/**
		 * Output a message about unsatisfactory version of PHP.
		 */
		public function notice_old_php_version(): void {
			$help_link = sprintf( '<a href="%1$s" target="_blank" rel="noopener noreferrer">%1$s</a>', 'https://wordpress.org/about/requirements/' );

			$message = sprintf(
			// Translators: 1: plugin display name, 2: required minimum PHP version, 3: current PHP version, help link
				_x(
					'%1$s requires at least PHP version %2$s in order to work. You have version %3$s. Please see %4$s for more information.',
					'notice requires newer PHP version',
					'rankbear'
				),
				'<strong>' . PluginData::get_plugin_display_name() . '</strong>',
				'<strong>' . PluginData::required_min_php_version() . '</strong>',
				'<strong>' . PHP_VERSION . '</strong>',
				$help_link
			);

			$this->do_admin_notice( $message );
		}

		/**
		 * Output a message that this plugin only works on live sites.
		 */
		public function notice_only_live_sites(): void {
			$message = sprintf(
			// Translators: %s: plugin display name
				_x(
					'%s only works on live/production domains, and this domain was detected as a testing/staging/development site. No need to worry; things will work again once you are running this plugin on your live site. Until then, this plugin will not run any of its code.',
					'notice only for production sites',
					'rankbear'
				),
				'<strong>' . PluginData::get_plugin_display_name() . '</strong>'
			);

			$this->do_admin_notice( $message, 'warning' );
		}

		/**
		 * Output a wp-admin notice.
		 *
		 * @param string $message
		 * @param string $type
		 */
		public function do_admin_notice(
			string $message,
			string $type = 'error'
		): void {
			$class = sprintf(
				'%s %s',
				$type,
				sanitize_html_class( PluginData::plugin_text_domain() )
			);

			printf( '<div class="notice notice-%s"><p>%s</p></div>', $class, $message );
		}

		/**
		 * Check if we have everything that is required.
		 *
		 * @return bool
		 */
		public function is_ready(): bool {
			$success = true;

			if ( version_compare(
				PHP_VERSION,
				PluginData::required_min_php_version(),
				'<'
			) ) {
				add_action( 'admin_notices', [ $this, 'notice_old_php_version' ] );
				$success = false;
			}

			$freemius = ( new RbFreemius() )->freemius_global();

			if ( empty( $freemius ) ) {
				$success = false;
			} else {
				$f_site = $freemius->get_site();
			}

			// https://freemius.com/help/documentation/selling-with-freemius/license-utilization/
			if (
				$success
				&& ! empty( $f_site )
				&& $f_site->is_localhost()
			) {
				//add_action( 'admin_notices', [ $this, 'notice_only_live_sites' ] );
				//$success = false;
			}

			return $success;
		}

		/**
		 * Get the file path of the plugin file from the plugin slug, if the plugin is installed.
		 *
		 * @param string $slug Plugin slug (typically folder name) as provided by the developer.
		 *
		 * @return string Either file path for plugin directory, or just the plugin file slug.
		 */
		private function get_plugin_basename_from_slug( string $slug ): string {
			$keys = array_keys( get_plugins() );

			foreach ( $keys as $key ) {
				if ( preg_match( '|^' . $slug . '/|', $key ) ) {
					return $key;
				}
			}

			return $slug;
		}

	}
}