<?php

namespace RankBear\Common;

use RankBear\Common\Utilities\Posts as Posts;
use RankBear\Common\Utilities\Strings as Strings;
use RankBear\PluginData as PluginData;
use WP_Post;
use WP_Post_Type;
use WP_Query;
use WP_REST_Request;

// Abort if this file is called directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( RestApi::class ) ) {
	/**
	 * Custom REST API endpoint and supporting code.
	 *
	 * @link https://developer.wordpress.org/rest-api/extending-the-rest-api/adding-custom-endpoints/
	 */
	class RestApi {

		/**
		 * @var string
		 */
		static $api_version = 'v1';

		/**
		 * @var Strings
		 */
		var $strings;

		public function __construct() {
			$this->strings = new Strings();
		}

		/**
		 * Get the namespace for our own REST route(s).
		 *
		 * Should not start or end with a slash (PHP Notice as of WP v5.4.2).
		 * Example: `rankbear/v1`
		 *
		 * @see register_rest_route() For use by this function.
		 *
		 * @return string
		 */
		private static function get_namespace() {
			return sprintf(
				'%s/%s',
				PluginData::plugin_text_domain(),
				self::$api_version
			);
		}

		/**
		 * Register our endpoint's route(s).
		 */
		public function register_rest_routes() {
			register_rest_route(
				self::get_namespace(),
				'public-posts', [
					'methods'             => 'GET',
					'permission_callback' => function () {
						return is_user_logged_in();
					},
					'callback'            => [ $this, 'get_all_public_posts' ],
					'args'                => [
						'id'       => [
							'validate_callback' => function ( $param, $request, $key ) {
								return 0 < intval( $param );
							},
						],
						'page'     => [
							'validate_callback' => function ( $param, $request, $key ) {
								return 0 < absint( $param );
							},
						],
						'per_page' => [
							'validate_callback' => function ( $param, $request, $key ) {
								return is_numeric( $param );
							},
						],
						'search'   => [
							'validate_callback' => function ( $param, $request, $key ) {
								return '' !== esc_html( $param );
							},
						],
					],
				]
			);

			register_rest_route(
				self::get_namespace(),
				'public-post-from-id-or-url', [
					'methods'             => 'GET',
					'permission_callback' => function () {
						return is_user_logged_in();
					},
					'callback'            => [ $this, 'get_public_post_from_id_or_url' ],
					'args'                => [
						'id'  => [
							'validate_callback' => function ( $param, $request, $key ) {
								return 0 < intval( $param );
							},
						],
						'url' => [
							'validate_callback' => function ( $param, $request, $key ) {
								return esc_url( $param );
							},
						],
					],
				]
			);
		}

		/**
		 * All public posts that are Published or Future Scheduled, regardless of post type.
		 *
		 * @param WP_REST_Request $request
		 *
		 * @return array
		 */
		public function get_all_public_posts( WP_REST_Request $request ) {
			$args = [
				'post_type'           => array_keys( ( new Posts() )->get_public_post_types() ),
				'post_status'         => [ 'publish', 'future' ], // Add 'future' & don't allow WP to auto-add 'private' because user is logged-in.
				'ignore_sticky_posts' => true,
			];

			// Post ID
			$post_id = absint( $request->get_param( 'id' ) );
			if ( ! empty( $post_id ) ) {
				$args['p'] = $post_id;
			}

			// Page Number
			$paged = absint( $request->get_param( 'page' ) );
			if ( ! empty( $paged ) ) {
				$args['paged'] = $paged;
			}

			// Posts Per Page (default of 40, not 10)
			$per_page = absint( $request->get_param( 'per_page' ) );
			if ( ! empty( $per_page ) ) {
				$args['posts_per_page'] = $per_page;
			} else {
				$args['posts_per_page'] = 40; // Long enough list without having tiny scroll bar control.
			}

			// Search
			$search = trim( esc_html( $request->get_param( 'search' ) ) );

			if ( empty( $search ) ) {
				// If searching, allow results to be best match, else let's sort by post title from A to Z.
				$args['order']   = 'ASC';
				$args['orderby'] = 'title';
			} else {
				/**
				 * @see \WP_Query::parse_search() Searches post Title, Excerpt, and Content.
				 */
				$args['s'] = $search;
			}

			/**
			 * The array to send to WP_Query for the REST API request for public posts.
			 *
			 * @since 1.0.0
			 *
			 * @param array $args The array to send to WP_Query for the REST API request for public posts.
			 *
			 * @return array
			 */
			$args = (array) apply_filters( PluginData::plugin_text_domain_underscores() . '_' . __FUNCTION__ . '_args', $args );

			$posts = ( new WP_Query( $args ) )->get_posts();

			$result = [];

			foreach ( $posts as $post ) {
				$value = $this->get_single_post_response_data( $post );
				if ( ! empty( $value ) ) {
					$result[] = $value;
				}
			}

			return rest_ensure_response( $result );
		}

		/**
		 * Given a URL, get its Post ID (but only if the post is public).
		 *
		 * Also works if path with leading slash -- URL lacking home_url() -- is sent.
		 *
		 * @param WP_REST_Request $request
		 *
		 * @return array
		 */
		public function get_public_post_from_id_or_url( WP_REST_Request $request ) {
			$post_id = $request->get_param( 'id' );

			if ( empty( $post_id ) ) {
				$post_id = url_to_postid( $request->get_param( 'url' ) );
			}

			$result = $this->get_single_post_response_data( $post_id );

			return rest_ensure_response( $result );
		}

		/**
		 * Get a consistent data response for single post responses.
		 *
		 * ONLY WORKS FOR POSTS OF A PUBLICLY-QUERYABLE POST TYPE.
		 *
		 * @param int|WP_Post $post Post ID or object.
		 *
		 * @return array Empty array if invalid Post.
		 */
		public function get_single_post_response_data( $post ): array {
			if ( is_numeric( $post ) ) {
				$post = get_post( $post );
			}

			if ( ! $post instanceof WP_Post ) {
				return [];
			}

			$post_type = get_post_type_object( $post->post_type );

			// Unexpected since a post was found but just to be sure.
			// We don't enforce public, publicly_queryable, show_in_rest, etc. here because this is just for prepping data for selection logic upstream.
			if ( ! $post_type instanceof WP_Post_Type ) {
				return [];
			}

			$title = get_the_title( $post->ID );

			$title_enhanced = sprintf(
				'%s [%s #%d]',
				$title,
				$post_type->label,
				$post->ID
			);

			$short = $this->strings->stringy( $title )->truncate( 40, '…' );

			$short_enhanced = sprintf(
				'%s [%s #%d]',
				$short,
				$post_type->label,
				$post->ID
			);

			return [
				'id'                 => $post->ID,
				'permalink'          => get_permalink( $post->ID ),
				'slug'               => $post->post_name,
				'title'              => $title, // NOT truncated
				'titleEnhanced'      => $title_enhanced, // NOT truncated with post type and post ID
				'titleShort'         => $short, // truncated
				'titleShortEnhanced' => $short_enhanced, // truncated with post type and post ID
				'type'               => $post_type->label,
			];
		}

	}
}