<?php

namespace RankBear\Core;

use RankBear\Admin as Admin;
use RankBear\Common as Common;
use RankBear\PluginData as PluginData;

// Abort if this file is called directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( Init::class ) ) {
	/**
	 * The core plugin class.
	 * Defines internationalization, admin-specific hooks, and public-facing site hooks.
	 */
	class Init {

		/**
		 * The loader that's responsible for maintaining and registering all hooks that power
		 * the plugin.
		 *
		 * @var      Loader $loader Maintains and registers all hooks for the plugin.
		 */
		protected $loader;

		/**
		 * Initialize and define the core functionality of the plugin.
		 */
		public function __construct() {
			$this->load_dependencies();
			$this->define_common_hooks();
			$this->define_admin_hooks();
			//$this->define_public_hooks();
		}

		/**
		 * Loads the following required dependencies for this plugin.
		 *
		 * - Loader - Orchestrates the hooks of the plugin.
		 * - Admin - Defines all hooks for the admin area.
		 * - Frontend - Defines all hooks for the public side of the site *** we don't have any ***
		 */
		private function load_dependencies(): void {
			$this->loader = new Loader();
		}

		/**
		 * Register all the hooks related to both the admin area and the public-facing functionality of the plugin.
		 */
		private function define_common_hooks(): void {
			// Settings Fields must not be behind an `is_admin()` check, since it's too late.
			$settings = new Common\Settings\Main();

			/**
			 * If we don't use the 'rest_api_init' hook, things like getting public post types won't include custom
			 * post types, even if 'show_in_rest' is true. This is also why registering the settings cannot be called
			 * only if is_admin().
			 */
			$this->loader->add_action( 'admin_init', $settings, 'register_settings' );
			$this->loader->add_action( 'rest_api_init', $settings, 'register_settings' );

			$rest_api = new Common\RestApi();
			$this->loader->add_action( 'rest_api_init', $rest_api, 'register_rest_routes' );
		}

		/**
		 * Register all the hooks related to the admin area functionality of the plugin.
		 * Also works during Ajax.
		 */
		private function define_admin_hooks(): void {
			$common = new Common\Common();

			if ( ! $common->current_request_is( 'admin' ) ) {
				return;
			}

			$assets = new Admin\Assets();

			// Register the plugin's admin assets.
			$this->loader->add_action( 'init', $assets, 'register_styles' );
			$this->loader->add_action( 'init', $assets, 'register_scripts' );

			// Enqueue the Meta Box assets.
			$this->loader->add_action( 'admin_enqueue_scripts', $assets, 'enqueue_meta_box_assets' );

			// Admin menu
			$this->loader->add_action( 'admin_menu', $assets, 'add_plugin_admin_menu' );

			$main = new Admin\Main();

			// Plugin action links
			$this->loader->add_filter(
				'plugin_action_links_' . PluginData::plugin_basename(),
				$main,
				'customize_action_links'
			);

			// Admin page <head> title on Settings Page.
			$this->loader->add_action( 'admin_title', $main, 'override_settings_page_title' );

			// Post Editor Meta Box (both Block and Classic Editors)
			$this->loader->add_action( 'add_meta_boxes', $main, 'add_meta_box' );
		}

		/**
		 * Register all the hooks related to the public-facing (is not admin or is Ajax) functionality of the plugin.
		 */
		private function define_public_hooks(): void {
			$common = new Common\Common();

			if ( ! $common->current_request_is( 'frontend' ) ) {
				return;
			}
		}

		/**
		 * Run the loader to execute all the hooks with WordPress.
		 */
		public function run(): void {
			$this->loader->run();
		}

		/**
		 * The reference to the class that orchestrates the hooks with the plugin.
		 *
		 * @return Loader Orchestrates the hooks of the plugin.
		 */
		public function get_loader(): Loader {
			return $this->loader;
		}
	}
}
