<?php

declare( strict_types=1 );

namespace RankBear;

use RankBear\Common\Settings\Main as Common_Main;
use WP_Error;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( RbApi::class ) ) {
	/**
	 * RankBear API setup.
	 *
	 * Most API calls happen in JS but we still use this class to pass some data to JS to be able to do so.
	 *
	 * @link https://api.rankbear.com/api/documentation The API's HTML documentation.
	 * @link https://api.rankbear.com/docs The API's JSON documentation.
	 */
	class RbApi {

		/**
		 * @var Common_Main
		 */
		private $common_main;

		/**
		 * The WordPress option key for getting/setting the RankBear Site ID.
		 *
		 * @var string
		 */
		public $site_id_key = 'site_id';

		/**
		 * RbApi constructor.
		 */
		public function __construct() {
			$this->common_main = new Common_Main();
		}

		/**
		 * The base URL (including trailing slash) with which to prepend endpoints to generate the full API URL.
		 *
		 * @return string With trailing slash.
		 */
		public static function get_base_url(): string {
			return 'https://api.rankbear.com/api/';
		}

		/**
		 * Get an endpoint's request URL.
		 *
		 * @param string $endpoint
		 *
		 * @return string
		 */
		public static function get_request_url( string $endpoint ): string {
			return self::get_base_url() . trim( $endpoint );
		}

		/**
		 * Build the array of request arguments to send to the API.
		 *
		 * @param array $body
		 *
		 * @return array Empty if $body was unable to be JSON encoded so make sure to check for that.
		 */
		private static function get_request_args( array $body = [] ): array {
			$args = [
				'headers'     => [
					'Accept'       => 'application/json',
					'Content-Type' => 'application/json',
				],
				'httpversion' => '1.1',
			];

			if ( ! empty( $body ) ) {
				$body = json_encode( $body );

				if ( ! empty( $body ) ) {
					$args['body'] = $body;
				}
			}

			return $args;
		}

		/**
		 * Get the WordPress site URL.
		 *
		 * @see wp_allowed_protocols() We only allow HTTP or HTTPS.
		 *
		 * @return string
		 */
		public function get_site_url(): string {
			$url = apply_filters(
				PluginData::plugin_text_domain_underscores() . '_' . __FUNCTION__,
				get_home_url()
			);

			return esc_url_raw( $url, [ 'http', 'https' ] );
		}

		/**
		 * Get this WordPress site's RankBear API Site ID from stored option, GET request, or POST request.
		 *
		 * @return int|null Integer greater than zero if successful, else null.
		 */
		public function get_site_id(): ?int {
			$site_id = (int) $this->common_main->get_option( $this->site_id_key );

			if ( 0 < $site_id ) {
				return $site_id;
			}

			$site_id = $this->get_site_id_from_api();

			if ( empty( $site_id ) ) {
				$site_id = $this->create_site_at_api();
			}

			if ( empty( $site_id ) ) {
				return null;
			} else {
				$option = $this->common_main->get_prefixed_option_key( $this->site_id_key );

				// Should be true.
				$updated = update_option( $option, $site_id );

				return $site_id;
			}
		}

		/**
		 * Parse the API response into a Site ID.
		 *
		 * @param WP_Error|array $response
		 *
		 * @return int Zero if unsuccessful, else positive integer.
		 */
		private function get_site_id_from_api_response( $response ): int {
			if (
				$response instanceof WP_Error
				|| 200 !== wp_remote_retrieve_response_code( $response )
			) {
				return 0;
			}

			$response = wp_remote_retrieve_body( $response );

			$response = json_decode( $response );

			if (
				! is_object( $response )
				|| empty( $response->id )
				|| ! is_numeric( $response->id )
			) {
				return 0;
			}

			$response = (int) $response->id;

			return $response;
		}

		/**
		 * Get this WordPress site's existing API Site ID.
		 *
		 * @see \WP_Http::request()
		 *
		 * @return int Zero if unsuccessful, else positive integer.
		 */
		private function get_site_id_from_api(): int {
			$site_url = $this->get_site_url();

			// Unexpected but just in case.
			if ( empty( $site_url ) ) {
				return 0;
			}

			$request_url = self::get_request_url( 'get_site' );

			$request_url = add_query_arg( 'url', $site_url, $request_url );

			$response = wp_safe_remote_get( esc_url_raw( $request_url ), self::get_request_args() );

			return $this->get_site_id_from_api_response( $response );
		}

		/**
		 * Generate a new API Site ID for this WordPress site.
		 *
		 * @see \WP_Http::request()
		 *
		 * @return int Zero if unsuccessful, else positive integer.
		 */
		private function create_site_at_api(): int {
			$site_url = $this->get_site_url();

			// Unexpected but just in case.
			if ( empty( $site_url ) ) {
				return 0;
			}

			$body = [
				'name' => get_bloginfo( 'name' ),
				'url'  => $site_url,
			];

			$args = self::get_request_args( $body );

			$request_url = self::get_request_url( 'create_site' );

			$response = wp_safe_remote_post( $request_url, $args );

			return $this->get_site_id_from_api_response( $response );
		}
	}
}