<?php

declare( strict_types=1 );

namespace RankBear;

use Freemius;
use Freemius_Exception;
use RankBear\Admin\Assets as Assets;
use RankBear\PluginData as PluginData;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( RbFreemius::class ) ) {
	/**
	 * The class for Freemius' global instance and our customizations to it.
	 */
	class RbFreemius {

		/**
		 * Get our global Freemius instance. Set it if it does not yet exist.
		 */
		public function freemius_global(): ?Freemius {
			global $rankbear_freemius;

			if ( ! $rankbear_freemius instanceof Freemius ) {
				$this->init();
			}

			return $rankbear_freemius;
		}

		/**
		 * Set the global Freemius instance and, if successful, add some customizations.
		 *
		 * Fires an action hook regardless of whether the global was successfully set.
		 */
		private function init(): void {
			global $rankbear_freemius;

			if ( ! $rankbear_freemius instanceof Freemius ) {
				require_once PluginData::plugin_dir_path() . 'vendor/freemius/wordpress-sdk/start.php';

				$settings = new Common\Settings\Main();

				try {
					$rankbear_freemius = fs_dynamic_init(
						[
							'id'                  => '5213',
							'slug'                => PluginData::plugin_text_domain(),
							'type'                => 'plugin',
							'public_key'          => 'pk_45dc05df1d965bf30b93877fd41c9',
							'enable_anonymous'    => false, // without a Freemius account, we can't setup your site to our SaaS
							'is_premium'          => false,
							'has_premium_version' => false, // If your plugin is a serviceware, set this option to false.
							'has_addons'          => false,
							'has_paid_plans'      => true,
							'menu'                => [
								'slug'       => $settings->get_settings_page_slug(),
								'first-path' => $settings->get_main_settings_page_url_base(),
							],
							'navigation'          => 'tabs',
						]
					);
				}
				catch ( Freemius_Exception $e ) {
					printf(
					// Translators: %s: the Freemius Exception's message.
						esc_html_x(
							'Yikes! Freemius experienced an error: %s',
							'Freemius try/catch exception',
							'rankbear'
						),
						$e->getMessage()
					);
				}
			}

			if ( $rankbear_freemius instanceof Freemius ) {
				$this->do_setup();

				do_action( PluginData::plugin_text_domain_underscores() . '_freemius_load_success' );
			} else {
				do_action( PluginData::plugin_text_domain_underscores() . '_freemius_load_fail' );
			}
		}

		/**
		 * Add our Freemius customizations.
		 */
		private function do_setup(): void {
			// Plugin Icon
			$this->freemius_global()->add_filter( 'plugin_icon', [ $this, 'get_the_plugin_icon' ] );

			// Permissions
			$this->freemius_global()->add_filter( 'permission_list', [ $this, 'permissions' ] );

			// Connect
			$this->freemius_global()->add_action( 'connect/after', [ $this, 'connect_after' ] );
		}

		/**
		 * Get our plugin icon SVG.
		 *
		 * @return string
		 */
		public function get_the_plugin_icon(): string {
			return Assets::admin_images_dir_path() . 'rankbear.svg';
		}

		/**
		 * Add our custom permissions.
		 *
		 * The built-in permissions' priorities:
		 * Profile=5, Site=10, Notices=13, Events=20, Extensions=25
		 *
		 * @param array $permissions
		 *
		 * @return array
		 */
		public function permissions( array $permissions ): array {
			$freemius = $this->freemius_global();

			$permissions['support-system'] = [
				'icon-class' => 'dashicons dashicons-sos',
				'label'      => $freemius->get_text_inline( 'Customized Support System', 'support-system' ),
				'desc'       => $freemius->get_text_inline( "Such as loading Help Scout's beacon to give you quick access to our help articles.", 'permissions-help-scout' ),
				'priority'   => 30,
			];

			$permissions['newsletter'] = [
				'icon-class' => 'dashicons dashicons-email',
				'label'      => $freemius->get_text_inline( 'Join our newsletter', 'newsletter' ),
				'desc'       => $freemius->get_text_inline( 'Allow us to occasionally email you search engine ranking tips, product and service discounts, announcements, and other goodies. (No SPAM)', 'permissions-newsletter' ),
				'priority'   => 35,
				'optional'   => true,
			];

			return $permissions;
		}

		public function connect_after(): void {
			// Why we don't allow anonymous.
			$why_text_1 = esc_html_x(
				"RankBear is a Software as a Service (SaaS) that can track search engine rankings for as many keywords as you need for any publicly-accessible post on your site… and we need your site to connect to our servers via Freemius.",
				'Freemius connect text',
				'rankbear'
			);

			$why_text_2 = esc_html_x(
				"Our servers don't have separate user accounts to maintain, and there aren't any API keys to juggle. It's a quick and easy setup because we just use the Freemius connection to register your site, which is why this step is required to get started, even for the free keyword tracking plan. Let's get started!",
				'Freemius connect text',
				'rankbear'
			);

			// Our terms, too.
			$terms_text = sprintf(
			// Translators: 1: opening link tag with URL, 2: closing link tag
				_x(
					'In addition to Freemius\' terms, you\'re agreeing to %1$sRankBear\'s terms%2$s and data tracking permissions.',
					'Freemius connect text',
					'rankbear'
				),
				'<a href="https://rankbear.com/terms/" target="_blank" rel="noopener noreferrer">',
				'</a>'
			);

			printf(
				'<div class="notice notice-info" style="margin-left: 0; width: 480px;">
<p style="font-style: italic;">%1$s</p>
<p style="font-style: italic;">%2$s</p>
<p style="font-style: italic;">%3$s</p>
</div>',
				$why_text_1,
				$why_text_2,
				$terms_text
			);
		}

		/**
		 * Whether there is both a user account with Freemius plus tracking enabled, as is required by this SaaS plugin.
		 *
		 * @see Freemius::is_registered() Comments on this method is how we know this is what we needed to check.
		 *
		 * @return bool
		 */
		public function site_is_connected() {
			return (
				$this->freemius_global()->is_registered()
				&& $this->freemius_global()->is_tracking_allowed()
			);
		}
	}
}